var // 13 endogenous variables
lamda_hat,  % consumption
y_hat,      % actual output 
a_hat,      % technology
z_hat,      % technology
r_hat,      % interest rate
pii_hat,    % inflation rate
e_hat,      % cost-push shock
g_hat,      % growth rate of output
q_hat,      % the potential output
x_hat,      % output gap
y_obs,      % obs. real GDP per capita
pii_obs,    % obs. GDP deflator
r_obs;      % obs. short-term interest rate


varexo // 4 exogenous variables
epsa,       % preference shock
epse,       % cost-push shock
epsz,       % technology shock
epsr;       % monetary policy shock


parameters
// 12 parameters need to estimate
z,          % Output growth rate at the steady state
beta,       % Discount rate
pii,        % steady state of inflation
psi,        % Calvo parameter

// 10 parameters are estimated
gamma,      % consumption habit formation
alpha,      % price indexation
rhopii,     % persistent parameter in inflation in MP rule
rhog,       % persistent parameter in growth rate in MP rule
rhoa,       % persistent parameter for preference shock
rhoe,        % persistent parameter for cost-push shock
%rhor,      % persistent parameter in interest rate in MP rule
%rhox,      % persistent parameter in output gap in MP rule
eps_a,      % preference shock
eps_e,      % cost-push shock
eps_z,      % technology shock
eps_r      % monetary policy shock
;         
// assigning the parameters
// calibrating parameters
z= 1.0046;       % Output growth rate at the steady state
beta = 0.9987;   % Discount rate
pii=1.0062;      % Inflation at the steady state
gamma = 0.39;    % consumption habit formation
alpha= 0.000;    % price indexation
psi= 0.1;        % Calvo parameter
rhopii=1.5;      % persistent parameter in inflation in MP rule
rhog=0.5;        % persistent parameter in growth rate in MP rule
rhoa=0.85;       % persistent parameter for preference shock
rhoe=0.00;       % persistent parameter for cost-push shock
%rhox=0.3;       % persistent parameter in output gap in MP rule
%rhor=0.5 ;      % persistent parameter in interest rate in MP rule
eps_a =0.08;     % preference shock
eps_e =0.001 ;   % cost-push shock
eps_z= 0.009;    % technology shock
eps_r = 0.001 ;  % monetary policy shock
%-------------------------------------------------------------------------

model(linear);

// endogenous variables (the lineared system without shock process)

%1. Marginal utility of consumption 
(z-beta*gamma)*(z-gamma)*lamda_hat = gamma*z*y_hat(-1) - (z^2 + beta*gamma^2)*y_hat  + beta*gamma*z*y_hat(+1)+ (z-beta*gamma*rhoa)*(z - gamma)*a_hat - gamma*z*z_hat;
                              
                                     
%2. The New Keyesian IS curve
lamda_hat = r_hat + lamda_hat(+1) - pii_hat(+1);

%3. The New Keyesian PC curve
(1+beta*alpha)*pii_hat = alpha*pii_hat(-1) + beta*pii_hat(+1) - psi*lamda_hat +psi*a_hat + e_hat;

%4. Growth rate of output
g_hat = y_hat - y_hat(-1) + z_hat;

%5. Efficient level of output (the potential output)
gamma*z*q_hat(-1) - (z^2 + beta*gamma^2)*q_hat + beta*gamma*z*q_hat(+1) + beta*gamma*(z - gamma)*(1-rhoa)*a_hat - gamma*z*z_hat =0;
                           

%6. The output gap 
x_hat = y_hat - q_hat;


// exogenous variables (shock process)

%7. The law of motion for the preference shock
a_hat = rhoa*a_hat(-1) + epsa;

%8. The law of motion for the cost-push shock
e_hat = rhoe *e_hat(-1) + epse;

%9. The law of motion for the technology shock
z_hat = epsz;

// Central bank

%10. Monetary policy role
r_hat =r_hat(-1) + rhopii*pii_hat + rhog*g_hat + epsr;
%r_hat =rhor*r_hat(-1) + rhopii*pii_hat +rhox*x_hat + rhog*g_hat + epsr;

// The 3-equattion measurement system

%11. Matching output
g_hat  = y_obs - log(z);                                 % right hand side is demeaning data, log(z) = mean(y_obs)
//note: log(z) = mean(y_obs)

%12. Matching inflation
pii_hat = pii_obs- log(pii);                              % right hand side is demeaning data, log(pii) = mean(pii_obs)
//note: log(pii) = mean(pii_obs)

%13. Matching interest rate
r_hat = r_obs - (log(z) - log(beta) + log(pii));          % right hand side is demeaning data, (log(z) - log(beta) + log(pii)) = mean(r_obs). It is based on steady state
//note: (log(z) - log(beta) + log(pii)) = mean(r_obs)

end;

varobs y_obs pii_obs r_obs;



steady;
check;

shocks; 
% preference shock
var epsa = (eps_a)^2;
% cost-push shock
var epse = (eps_e)^2;
% technology shock
var epsz = (eps_z)^2;
% monetary policy shock
var epsr = (eps_r)^2;
end;

estimated_params; 

gamma,,,,       beta_pdf,0.4,0.1;         % consumption habit formation
alpha,,,,       beta_pdf,0.011,0.1;       % price indexation

// 4 parameters in MP rule
rhopii,,,,     beta_pdf,0.5,0.1;          % persistent parameter in inflation in MP rule
rhog,,,,     beta_pdf,0.12,0.1;           % persistent parameter in growth rate in MP rule
%rhox,,,,     beta_pdf,0.85,0.1;          % persistent parameter in output gap in MP rule
%rhor,,,,     beta_pdf,0.85,0.1;          % persistent parameter in interest rate in MP rule

// 2 persistent parameters in shock process
rhoa,,,,     beta_pdf,0.85,0.1;          % persistent parameter for preference shock
rhoe,,,,     beta_pdf,0.011,0.1;         % persistent parameter for cost-push shock

// 4 variances of shock
stderr epsa,,,,   inv_gamma_pdf,0.08,inf;        % preference shock
stderr epse,,,,   inv_gamma_pdf,0.002,inf;       % cost-push shock
stderr epsz,,,,   inv_gamma_pdf,0.009,inf;       % technology shock
stderr epsr,,,,   inv_gamma_pdf,0.001,inf;       % monetary policy shock

end;

estimation(
datafile=data,
plot_priors=0, % default =1
mode_compute=7,%4 (Chris Sim), 7 ( fminsearch Nelder-Mead 1965), 9 (Covariance Matrix Adaptation Evolution Strategy) algorithm of Hansen and Kern (2004))
%optim =('MaxIter',50000,'MaxFunEvals',50000),
%mh_replic=500000,      % THE MOST IMPORTANCE. Number of draws for each Makov chain. Default: 20,000
mh_nblocks = 1,   % THE MOST IMPORTANCE. Number of parallel chains for Metropolis-Hasting Algorithm (Default=2, very low). it should be at least 5
mh_drop = 0.3,    % the number of burn-in. default =0.5
mh_jscale = 0.3,  % THE MOST IMPORTANCE. The scale for jumping distribution in the MH algorithm (default =0.2, rarely satisfactory)
                  % this scale must be tunned to make sure the acceptance rate ranging btw 0.2 and 0.4
                  % when Ph set the jumping scale of 0.2 as a default number, then the acceptance ratio is
                  % over 0.4 (too high in comparison to literature only 0.2 and 0.4)
                  % that's why P increase the jump scale from default level of 0.2 to 0.3
                  % as a result, the acceptance ratio lies on the range btw 0.2 and 0.4
                  % the rule to adjust the jumping scale (c)
                  % if the acceptance ratio (R) is TOO LOW, then the jumping scale (c) should be REDUCED (WHY?)
                  % if the acceptance ratio (R) is TOO HIGH, then the jumping scale (c) should be INCREASED (WHY?)
mh_init_scale=1 , % Like An & Schorfheide (2007)to generate a starting value for MH


%load_mh_file,  % VERY USEFUL TO SPEED UP.By loading the previuos MH simulation
%mode_file=nkp_mode, % to use the previous likelihood result
mode_check_neighbourhood_size =1000,
//filtered_vars,
//filter_step_ahead=[1 2 1]
diffuse_filter, % to stationanize data
mode_check
//lik_init=1,      
//nograph=0
); 